package uicc.contactmanager;


/**
 * <p>
 *   A group list is a list of all groups defined for a contact manager.
 * </p>
 * <p>
 *   Within a group list, each group is assigned a unique numeric identifier.
 *   <br>
 *   Each group belongs to a unique group list.
 * </p>
 * <p>
 *   <code>GroupsListView</code> instances provide:
 *   <ul>
 *     <li>methods to add and remove a group from a group list;</li>
 *     <li>an enumeration scheme to retrieve the list of groups in a group list, either 
 *         matching filter criteria, or unfiltered;</li>
 *   </ul>
 * </p>
 * <p>
 *   The <code>GroupsListView</code> class also provides a factory method to create a <code>GroupView</code>
 *   instance.
 * </p>
 * <p>
 *   Multiple <code>GroupsListView</code> instances may be created by the same or different 
 *   applications. Although they may represent the same groups list, each of them has its 
 *   own state.
 *   <br>
 *   This particularly applies to the current enumeration progress, which is specific to
 *   each instance.
 * </p>
*/
public interface GroupsListView {
    /**
     * <p>
     *   Factory method used to create a <code>GroupView</code> instance.
     * </p>
     * <p>
     *   The instance is initially created in the DESELECTED state.
     *   <br> 
     *   See the description of the <code>GroupView</code> interface for detailed information
     *   about its complete life cycle.
     * </p>
     * <p>
     *   The new instance is owned by the calling application.
     * </p>
     *
     * @return new <code>GroupView</code> instance
     */
    public GroupView createGroupViewInstance();

    /**
     * <p>
     *   Starts a new enumeration of all groups in the list.
     * </p>
     * <ul>
     *   <li>A <code>GroupsListView</code> instance can process one enumeration at a time.<br>
     *       If an enumeration is already in progress, it is reset to the first group
     *       in the list.</li>
     *   <li>The enumeration is processed using the method <code>nextGroup()</code>.</li>
     * </ul>
     */
    public void startEnumeration();

    /**
     * <p>
     *   Stars the enumeration of all groups in the groups list, that contain a specific contact.
     * </p>
    * <p>
     *   In addition to the filtering mechanism described here, all requirements of the method 
     *   <code>startEnumeration()</code> apply to this method.
     * </p>
     * @param contactIdentifier identifier of the contact to look for
     */
    public void startEnumeration(int contactIdentifier);
    
    /**
     * <p>
     *   Checks if there are more groups in the current group enumeration.
     * </p>
     * 
     * @return <code>true</code> if there is at least one more group in the current enumeration
     */
    public boolean hasMoreGroups();

    /**
     * <p>
     *   Binds a <code>GroupView</code> instance with the next group in the current enumeration.
     * </p>
     * <p>
     *   The order in which the groups are enumerated is unspecified.
     * </p>
     * <p>
     *   After successful operation, the <code>GroupView</code> instance is in the SELECTED state, 
     *   and is bound to the next group in the current enumeration.
     * </p>
     * <p>
     *   If the current enumeration has not been started, or already reached its end,
     *   or after unsuccessful operation for any reason, the <code>GroupView</code> instance
     *   is in DESELECTED state.
     * </p>
     * 
     * @param groupView <code>GroupView</code> instance to bind to the next group in the enumeration.
     * 
     * @return group identifier, or -1 if the current enumeration reached its end.
     */
    public int selectNextGroup(GroupView groupView);

    /**
     * <p>
     *   Binds a <code>GroupView</code> instance to a group that matches the provided 
     *   <code>groupIdentifier</code>.
     * </p>
     * <p>
     *   After successful operation, the <code>GroupView</code> instance is in SELECTED state, 
     *   and is bound to the selected group.
     * </p>
     * <p>
     *   After unsuccessful operation, the <code>GroupView</code> instance is in DESELECTED state.
     * </p>
     * 
     * @param groupView <code>GroupView</code> instance to bind with the next group that
     *                matches the identifier. 
     * @param groupIdentifier group identifier of group to get
     * 
     * @throws <code>uicc.contactmanager.ContactManagerException</code>:
     *    <ul>
     *      <li>with reason <code>WRONG_IDENTIFIER</code> if the list contains no group with the 
     *          <code>groupIdentifier</code> provided.</li>
     *    </ul>
     */
    public void selectGroup(GroupView groupView, int groupIdentifier);

    /**
     * <p>
     *   Creates a new group and adds it to the groups list.
     * </p>
     * <p>
     *   The new group is set with the provided group name, in a way that is similar to a call to 
     *   method <code>GroupView.setGroupName()</code> with the parameters <code>name</code>, <code>nameOffset</code>
     *   and <code>nameLength</code>.
     * </p>
     * <p>
     *   When a group is added to a groups list, any enumeration over the same groups list currently 
     *   in progress may or may not include the new group, depending on the implementation.
     * </p>
     *
     * @param name name of the group to set
     * @param nameOffset offset in name buffer
     * @param nameLength length of name buffer
     * 
     * @return the new group's identifier
     */
    public int addGroup(byte [] name, short nameOffset, short nameLength);
    
    /**
     * <p>
     *   Removes a group from the groups list.
     * </p>
     * <p>
     *   This method fails in the following cases:
     *   <ul>
     *     <li>
     *       If any <code>GroupView</code> instance is currently bound to the removed group.</li>
     *     <li>
     *       If an enumeration is currently in progress in any instance of <code>GroupsListView</code>
     *       related the same contact manager, other than the method's target.
     *     </li>
     *   </ul>
     * </p>
     * 
     * @param groupIdentifier identifier of the group to be removed from the list
     * 
     * @throws <code>uicc.contactmanager.ContactManagerException</code> with the following reasons:
     *   <ul>
     *      <li><code>WRONG_IDENTIFIER</code> if <code>groupIdentifier</code>
     *          does not identify any group in the groups list.</li>
     *      <li><code>ITEM_SELECTED</code> if the group to be removed is currently selected by 
     *          a <code>GroupView</code> instance.</li>
     *      <li><code>ENUMERATION_IN_PROGRESS</code> if a conflicting enumeration is
     *          currently in progress.</li>
     *   </ul>
     */
    public  void removeGroup(int groupIdentifier);
    
    /**
     * <p>
     *   Returns the number of groups in the groups list.
     * </p>
     * 
     * @return number of groups in the group list
     */
    public int getNumberOfGroups();
}
